<?php
/*
 * @author Ralph Bolton, July 2004, www.coofercat.com
 * @version 1.0
 * @copyright Copyright &copy; 2004 Ralph Bolton.
 * @license Tiki License
 * @see http://tikiwiki.org for Licensing information.
*/

// Initialisation
// We can't use tikilib for this - it runs all kinds of stuff,
// most notibly the modules. That means they'll miss out on a
// proper environment, so won't get the query string and such like.
// Instead, we have to construct our environment as we need it.
// This is quicker, and any PHP we run will do it's own setup anyway.
require_once('lib/init/initlib.php');
require_once('db/tiki-db.php');
require_once('lib/tikidblib.php');
require_once('HTTP/Request.php');
require_once('lib/urlt/urltlib.php');

/*
 * string urlt_perform_regexes(string url)
 * This function takes a URL (possibly relative or fully qualified,
 * of any scheme) and runs a series of configurable regexes on it.
 * The regexes have a flag associated with them that can cut short
 * processing. That is, stop processing any more regexes if a match
 * was found or not found, for example. What ever result is obtained
 * is returned.
 *
 * Regexes are found in the tiki_urlt_regex_in database table.
 *
 * @access private
*/
function perform_regexes($url)
{
	#error_log("URL to transform is $url");

	// This comes to us via the libraries included above
	global $dbTiki;

	// Get the cache time from the config. We use a short cache
	// here to reduce the (plentiful) requests to the DB.
	$cache_time=1;
	if(($rs=$dbTiki->CacheExecute(1,"select config_value_int from tiki_urlt_config where config_key='urlt_in_cache_time'")) === false)
	{
		// Failed to read the DB - should log this?
		return $url;
	}
	else
	{
		$value=$rs->fields['config_value_int'];
		if(isset($value))
		{
			$cache_time=$value;
		}
	}

	// ADOdb style DB access.
	// Use whatever cache time the user specified. This can really help performance!
	if(($rs=$dbTiki->CacheExecute($cache_time,"select * from tiki_urlt_regex_in order by regexid"))===false)
	{
		// Failed to read the database properly.
		error_log("urlt_in: Failed to Read database: " . $dbTiki->ErrorMsg());
		return $url;
	}
	while ($array = $rs->FetchRow())
	{
		// First, see if the rule is disabled - if so, skip it.
                if($array['continue_flag'] == 3)
                {
                        // Rule disabled - skip it
                        //error_log("Skipping disabled rule " . $array['regexid']);
                        continue;
                }

		// We have to check first, replace second. We can't just attempt the replace
		// because there's no way to know if it was successful if you don't change
		// anything (which is possible - and likely!)
		//if($newurl == $url)
		if(!preg_match('/'.$array['regex_left'].'/'.$array['regex_flags'],$url))
		{
			# No change - no match!
			if($array['continue_flag'] == 2)
			{
				//error_log("Ceasing processing because flag = 2 (stop on no match)");
				break;
			}
		}
		else
		{
			# Changed - Match
			//$url=$newurl;
			$url=preg_replace('/'.$array['regex_left'].'/'.$array['regex_flags'], $array['regex_right'], $url);
			if($array['continue_flag'] == 1)
			{
				//error_log("Ceasing processing because flag = 1 (stop on match)");
				break;
			}
			elseif($array['continue_flag'] == 4)
			{
				header("Location: $url");
				die;
			}
		}
	}

	//error_log("Transformed URL is $url");
	return $url;
}

// Get going...

// URLt is not implemented as a "Tiki feature" yet. When it is, reinstate this code.
//if ($feature_urlt != 'y') {
//      $smarty->assign('msg', tra("This feature is disabled").": feature_urlt");
                                                                                                                             
//      $smarty->display("error.tpl");
//      die;
//}

// See if the user has enabled us...
$enabled= urlt_get_config_value($dbTiki,'urlt_in_enabled');
if((isset($enabled)) && ($enabled != 1))
{
	// Disabled. We dont' have smarty, so can't do a nice error message.
	// There's not a great deal of point having a configurable error for this
	// because end-users shouldn't ever be subjected to it!
	// We have to do this cheesy thing instead...
	print "URLt: Incoming URLt has been disabled\n";
	die;
}

// First, get the URL that we're translating. Normally this would be REQUEST_URI
// however, we allow an override for development/debugging/practice purposes
// The user can call tiki-urlt.php?url=... as well.
$url="";

if(isset($_REQUEST['url']))
{
	$url=$_REQUEST['url'];
}
else
{
	$url=$_SERVER['REQUEST_URI'];
}

# We have the URL to translate, so translate it...
$newurl=perform_regexes($url);

//error_log("Transformed $url into $newurl");

// We have to be careful to avoid request loops. That is, if we make a
// request to ourselves, it'll go around and around for ever. So, check to
// make sure we're not the object of their affections ;-)
// This probably shouldn't be hardcoded like this, but $_SERVER['SCRIPT_NAME']
// is only right if we're called by an ErrorDocument (mod_rewrite doesn't work)
if(preg_match('/tiki-urlt\.php/',$newurl))
{
	// Request loop!
	// We don't have smarty, so can't call a nice looking error. There's
	// really nothing useful that can be done though, so whilst it's a bit
	// nasty, it's about all we can do.
	header('HTTP/1.0 500 SERVER ERROR');
	header('Status: 500 SERVER ERROR');
	// See if there's an "error" page configured in...
        $error_page=urlt_get_config_value($dbTiki,'urlt_in_error_page','text');
        if((isset($error_page)) && ($error_page != ""))
        {
                // There is an error page, start using it...
                $newurl=$error_page;
                $newuri=preg_replace('/(\?|#).*$/','',$newurl);
                $errorfile=$_SERVER['DOCUMENT_ROOT'] . $newuri;
                if(!file_exists($errorfile))
                {
                        // Error page doesn't exist!
                        // Return a nasty error message ;-)
                        print "Tiki URLt: Request loop: $url translated to $newurl.\n";
                        print "Additionally, the configured error document ($error_page : $errorfile) was not found.\n";
                        die;
                }
                // Use the file!
                $file=$errorfile;
        }

	print "Tiki URLt: Request loop: $url translated to $newurl\n";
	print $_SERVER['SCRIPT_NAME'];
	die;
}
else
{

	// Okay, we have a good looking URL - go fetch it...
	// This requires that we check this URL for an actual file. We then figure out
	// the MIME type of that file (unless it's a PHP) and then return it. If it's
	// a PHP, then we simply include it, causing it to execute. We have to frig
	// a few environmentals first though, so the PHP thinks everything is as it
	// would normally be.
	// Beware! URLs don't (necessarily) end in the file extension, e.g.:
	// /tiki/tiki-index.php?page=banana

	// Get the URI (ie the bit before the ? or #)
	$uri=preg_replace('/(\?|#).*$/','',$url);

	// Now see if the URI ends in a / - if so, then we artificially add
	// index.php onto the end - this should probably be configurable!
	if(preg_match('/\/$/',$uri))
	{
		# This should be configurable!
		$uri=$uri . "index.php";
	}
	
	// Also do the same with the new url. When including the file, we don't
	// want the query string (etc) on the URL
	$newuri=preg_replace('/(\?|#).*$/','',$newurl);
	
	//error_log("URLt: $url translated to $newurl");
	
	// Construct the filename
	// We should check to make sure there's no funny stuff going on here
	// (ie. reading things outside the document root). NOT IMPLEMENTED!
	$file=$_SERVER['DOCUMENT_ROOT'] . $newuri;
	
	// Now check the file exists. If not, we have to return a 404 error.
	if(!file_exists($file))
	{
		// File doesn't exist. Return a 404 error
		// Set the outgoing result header...
		header('HTTP/1.0 404 NOT FOUND');
		header('Status: 404 NOT FOUND');
	
		// See if there's a "notfound" page configured in...
		$error_page=urlt_get_config_value($dbTiki,'urlt_in_notfound_page','text');
		if((isset($error_page)) && ($error_page != ""))
		{
			// There is an error page, start using it...
			$newurl=$error_page;
			$newuri=preg_replace('/(\?|#).*$/','',$newurl);
			$errorfile=$_SERVER['DOCUMENT_ROOT'] . $newuri;
			if(!file_exists($errorfile))
			{
				// Error page doesn't exist!
				// Return a nasty error message ;-)
				print "Tiki URLt: $url translated to $file which was not found.\n";
				print "Additionally, the configured error document ($error_page : $errorfile) was not found.\n";
				die;
			}
			// Use the file!
			$file=$errorfile;
		}
		else
		{
			// Instead, just return a weedy looking error message.
			// We dont' have Smarty, so can't have a nice looking error page.
			// This is nasty, but all that's left to do!
			print "Tiki URLt: $url translated to $file which was not found\n";
			die;
		}
	}
	else
	{
		// Either way, to have got here, the file was found. That means we should set
		// a status code of 200 - we may have been called as an error document, which
		// would mean the server would currently be ready to return 404. If we were
		// called as a result of Apache mod_rewrite, this has no effect.
		header('HTTP/1.0 200 OK');
		header('Status: 200 OK');
	}
}

// Finally!
// Now see if the file is PHP. If so, include it (which will run it) in a suitably frigged
// environment. If not, then see if we've got it's mime type configured, and return it
// with that.
$extension=strtolower(preg_replace('/^(.*)\.([^\.]+)$/','$2',$newuri));
if($extension == "php")
{
	// Is a PHP file... Include it...
	$file=$_SERVER['DOCUMENT_ROOT'] . $newuri;
	// Now frig the environment ready for the PHP...
	// Setup everything so the included PHP doesn't know it's not been called directly
	$_SERVER['DOCUMENT_URI']=$newurl;
	if(strlen($newurl) > strlen($newuri))
	{
		// There is some kind of query string...
		// Snip it off, discard the ?
		$qs=substr($newurl,strlen($newuri)+1);
		// Since regexes arn't great at checking to see if we've got
		// two ? in the query string, we "fix" this here, by replacing
		// any ? with &. That makes any appended query strings work
		// nicely together.
		$qs=str_replace('?','&',$qs);
		// Now parse the QS and set $_REQUEST and $_GET
		$parts=array();
		$parts=explode('&',$qs);
		foreach ($parts as $part)
		{
			if($part == "")
			{
				continue;
			}
			$keyvalue=array();
			$keyvalue=explode('=',$part,2);
			//error_log("QS: For part $part Got key=$keyvalue[0] value=$keyvalue[1]");
			$_GET[$keyvalue[0]]=$keyvalue[1];
			$_REQUEST[$keyvalue[0]]=$keyvalue[1];
			//error_log("QS: For part $part set value=" . $_REQUEST[$keyvalue[0]]);
		}
	}
	else
	{
		$qs="";
	}
	$_SERVER['QUERY_STRING']=$qs;
	$_SERVER['DOCUMENT_NAME']=preg_replace('/\/([^\/*])$/','$1',$newuri);
	//error_log("Including $file: qs=" . $_SERVER['QUERY_STRING'] );
	include($file);
	// Definitely do no more!
	die;
}
else
{
	// Non PHP - look up the MIME type and return the file
	// There's lots of scope here to do all sorts of good things. It'll all rely
	// on a mime type table in the database though.
	// Firstly, obviously, return non-PHP files (using correct mime type headers)
	// Secondly: Pass-Through HTML. The idea for this is that if you translate to
	// something that's text/html, you look up the config for PTH. If the file
	// is in a configured area, instead of just returning the HTML, you use it as
	// if it were a Tiki smarty template. In other words, the HTML gets wrapped
	// inside the Tiki page layout - neat huh? A handy way to migrate from HTML
	// to Tiki, and potentially useful for building web sites in it's own right.

	$config_key='extension_' . $extension;
	// See if we've got the MIME type configured for this extension...
	$mime=urlt_get_config_value($dbTiki,$config_key,'text');
	if($mime == "")
	{
		// No mime set, so see if we have a default...
		$mime=urlt_get_config_value($dbTiki,'urlt_in_default_mime','text');
		if((!isset($mime)) || ($mime == ""))
		{
			// No default set, use text/plain
			$mime='text/plain';
		}
	}
	// Use the mime type...
	header("Content-Type: $mime");
	// Return the file... (stream it in chunks - don't read it all in at once!)
	$handle=fopen($file,'r');
	if($handle)
	{
		while(($data=fgets($handle,8192)) !== false)
		{
			print $data;
		}
		fclose($handle);
	}
	else
	{
		// What to do here? We could probably return the error page, but
		// that's a pain as it means we've got to redo all the PHP enviroment
		// stuff (which could of course be in a function!)
		// Just return the error...
		print "URLt: Could not open $mime file $file\n";
	}
	die;
}

?>
